@@ -429,6 +429,11 @@ def launch_engine(
         "-s",
         help="Boot disk size in GB (default: 50GB, min: 20GB, max: 1000GB)",
     ),
+    availability_zone: Optional[str] = typer.Option(
+        None,
+        "--az",
+        help="Prefer a specific Availability Zone (e.g., us-east-1b). If omitted the service will try all public subnets.",
+    ),
 ):
     """Launch a new engine instance."""
     username = check_aws_sso()
@@ -471,6 +476,8 @@ def launch_engine(
         }
         if boot_disk_size is not None:
             request_data["boot_disk_size"] = boot_disk_size
+        if availability_zone:
+            request_data["availability_zone"] = availability_zone
 
         response = make_api_request("POST", "/engines", json_data=request_data)
 
@@ -679,9 +686,8 @@ def engine_status(
         status_lines.append("")
         status_lines.append("[bold]Attached Studios:[/bold]")
         for studio in attached_studios:
-            attach_time = studio.get("attach_time", "Unknown")
             status_lines.append(
-                f"  • {studio['user']} ({studio['studio_id']}) - attached {attach_time}"
+                f"  • {studio['user']} ({studio['studio_id']})"
             )
 
     console.print(Panel("\n".join(status_lines), title="Engine Status", border_style="blue"))
@@ -1262,20 +1268,26 @@ def create_ami(
     ec2 = boto3.client("ec2", region_name="us-east-1")
 
     try:
-        # First, we need to clean up the sentinel file via SSM
-        console.print("Cleaning up bootstrap sentinel file...")
+        # First, we need to clean up the instance before snapshotting
+        console.print("Cleaning up instance for AMI creation...")
         ssm = boto3.client("ssm", region_name="us-east-1")
 
+        # Clean up instance state, stopping SSM agent last to allow proper status reporting
+        cleanup_commands = [
+            "sudo rm -f /opt/dayhoff/first_boot_complete.sentinel",
+            "history -c",
+            "sudo rm -rf /tmp/* /var/log/messages /var/log/cloud-init.log",
+            "sudo rm -rf /var/lib/amazon/ssm/* /etc/amazon/ssm/*",
+            # Stop SSM agent last with a delay to allow status reporting
+            "sleep 2 && sudo systemctl stop amazon-ssm-agent &",
+        ]
+
         cleanup_response = ssm.send_command(
             InstanceIds=[engine["instance_id"]],
             DocumentName="AWS-RunShellScript",
             Parameters={
-                "commands": [
-                    "sudo rm -f /opt/dayhoff/first_boot_complete.sentinel",
-                    "history -c",
-                    "sudo rm -rf /tmp/* /var/log/*",
-                ],
-                "executionTimeout": ["60"],
+                "commands": cleanup_commands,
+                "executionTimeout": ["120"],
             },
         )
 
@@ -1290,9 +1302,14 @@ def create_ami(
             if result["Status"] in ["Success", "Failed"]:
                 break
 
-        if result["Status"] != "Success":
+        # Note: InProgress status is expected when SSM agent stops itself
+        if result["Status"] not in ["Success", "InProgress"]:
             console.print(
-                "[yellow]⚠️  Warning: Cleanup command may have failed[/yellow]"
+                f"[yellow]⚠️  Warning: Cleanup command status: {result['Status']}[/yellow]"
+            )
+        elif result["Status"] == "InProgress":
+            console.print(
+                "[dim]ℹ️  Cleanup command still in progress (expected when SSM agent stops itself)[/dim]"
             )
 
         # Get instance details to find volumes to exclude
@@ -1606,6 +1623,9 @@ def attach_studio(
     engines = response.json().get("engines", [])
     engine = resolve_engine(engine_name_or_id, engines)
 
+    # Flag to track if we started the engine in this command (affects retry length)
+    engine_started_now: bool = False
+
     if engine["state"].lower() != "running":
         console.print(f"[yellow]⚠️  Engine is {engine['state']}[/yellow]")
         if engine["state"].lower() == "stopped" and Confirm.ask(
@@ -1618,33 +1638,14 @@ def attach_studio(
                 console.print("[red]❌ Failed to start engine[/red]")
                 raise typer.Exit(1)
             console.print("[green]✓ Engine started[/green]")
-            console.print("Waiting for engine to be ready...")
-            # Wait until the engine is fully running & marked ready (max 5 min)
-            console.print("Waiting for engine to be ready (this can take a couple of minutes)…")
-            # Progress classes already imported at module level – do not re-import to avoid UnboundLocalError
-
-            with Progress(SpinnerColumn(), TimeElapsedColumn(), TextColumn("[progress.description]{task.description}"), transient=True) as prog:
-                prog.add_task("Awaiting engine readiness…", total=None)
-                max_attempts = 30  # 30 × 10 s  ≈ 5 min
-                for _ in range(max_attempts):
-                    time.sleep(10)
-
-                    status_resp = make_api_request("GET", f"/engines/{engine['instance_id']}")
-                    if status_resp.status_code != 200:
-                        continue
-
-                    detailed = status_resp.json().get("engine", {})
-                    state_ok = detailed.get("state", "").lower() == "running"
-                    ready_ok = detailed.get("ready") is True
-
-                    if state_ok and ready_ok:
-                        break
-                else:
-                    console.print("[yellow]Engine is still starting up. If attachment fails, wait a little longer and retry.[/yellow]")
+            # Mark that we booted the engine so attach loop gets extended retries
+            engine_started_now = True
+            # No further waiting here – attachment attempts below handle retry logic while the
+            # engine finishes booting.
         else:
             raise typer.Exit(1)
 
-    # Get SSH key
+    # Retrieve SSH public key (required for authorised_keys provisioning)
     try:
         public_key = get_ssh_public_key()
     except FileNotFoundError as e:
@@ -1653,36 +1654,121 @@ def attach_studio(
 
     console.print(f"Attaching studio to engine [cyan]{engine['name']}[/cyan]...")
 
-    with Progress(
-        SpinnerColumn(),
-        TextColumn("[progress.description]{task.description}"),
-        transient=True,
-    ) as progress:
-        task = progress.add_task("Attaching studio...", total=100)
+    # Determine retry strategy
+    max_attempts = 40 if engine_started_now else 3
+    retry_delay  = 10 if engine_started_now else 3
 
-        response = make_api_request(
-            "POST",
-            f"/studios/{studio['studio_id']}/attach",
-            json_data={
-                "vm_id": engine["instance_id"],
-                "user": target_user,  # Use target_user instead of username
-                "public_key": public_key,
-            },
-        )
+    if engine_started_now:
+        # Long spinner-based loop while the freshly started engine finishes booting
+        with Progress(
+            SpinnerColumn(),
+            TimeElapsedColumn(),
+            TextColumn("[progress.description]{task.description}"),
+            transient=True,
+        ) as prog:
+            task = prog.add_task("Attaching studio (engine is still booting)…", total=None)
 
-        progress.update(task, completed=100)
+            for attempt in range(max_attempts):
+                success, error_msg = _attempt_studio_attach(studio, engine, target_user, public_key)
+
+                if success:
+                    break  # success!
+
+                # Update spinner every 3rd try to avoid log spam
+                if attempt % 3 == 0:
+                    prog.update(task, description=f"Attaching studio (engine is still booting)… {attempt+1}/{max_attempts}")
+
+                if error_msg:
+                    console.print(f"[red]❌ Failed to attach studio: {error_msg}[/red]")
+                    return
+
+                time.sleep(retry_delay)
+
+            else:
+                console.print("[yellow]Engine is still starting up – please retry in a minute.[/yellow]")
+                return
+    else:
+        # Give the (already-running) engine a little breathing room – e.g. it may still be mounting EFS
+        max_attempts = 10  # ~1 min total
+        retry_delay  = 6
+
+        for attempt in range(max_attempts):
+            success, error_msg = _attempt_studio_attach(studio, engine, target_user, public_key)
+
+            if success:
+                break  # attached!
+
+            if error_msg:
+                # Fatal – bubble up immediately
+                console.print(f"[red]❌ Failed to attach studio: {error_msg}[/red]")
+                return
+
+            # Recoverable and still not ready – short wait + optional info
+            if attempt < max_attempts - 1:
+                console.print("[dim]Engine not ready yet – retrying …[/dim]")
+                time.sleep(retry_delay)
+
+        else:
+            console.print("[yellow]Engine is busy or still initialising – please retry in about a minute.[/yellow]")
+            return
+
+    # Successful attach path
+    console.print(f"[green]✓ Studio attached successfully![/green]")
+
+    # Update SSH config - use target_user for the connection
+    update_ssh_config_entry(engine["name"], engine["instance_id"], target_user)
+    console.print(f"[green]✓ SSH config updated[/green]")
+    console.print(f"\nConnect with: [cyan]ssh {engine['name']}[/cyan]")
+    console.print(f"Files are at: [cyan]/studios/{target_user}[/cyan]")
+
+
+def _attempt_studio_attach(studio, engine, target_user, public_key):
+    response = make_api_request(
+        "POST",
+        f"/studios/{studio['studio_id']}/attach",
+        json_data={
+            "vm_id": engine["instance_id"],
+            "user": target_user,
+            "public_key": public_key,
+        },
+    )
 
     if response.status_code == 200:
-        console.print(f"[green]✓ Studio attached successfully![/green]")
+        return True, None
 
-        # Update SSH config - use target_user for the connection
-        update_ssh_config_entry(engine["name"], engine["instance_id"], target_user)
-        console.print(f"[green]✓ SSH config updated[/green]")
-        console.print(f"\nConnect with: [cyan]ssh {engine['name']}[/cyan]")
-        console.print(f"Files are at: [cyan]/studios/{target_user}[/cyan]")
+    # --- determine if we should retry ---
+    recoverable = False
+    if response.status_code in (409, 503):
+        recoverable = True
     else:
-        error = response.json().get("error", "Unknown error")
-        console.print(f"[red]❌ Failed to attach studio: {error}[/red]")
+        err_msg = response.json().get("error", "").lower()
+        RECOVERABLE_PATTERNS = [
+            "not ready",
+            "still starting",
+            "initializing",
+            "failed to mount",
+            "device busy",
+            "not available",
+            "pending",  # VM state pending
+        ]
+        FATAL_PATTERNS = [
+            "in-use",
+            "already attached",
+            "permission",
+        ]
+        if any(p in err_msg for p in FATAL_PATTERNS):
+            recoverable = False
+        elif any(p in err_msg for p in RECOVERABLE_PATTERNS):
+            recoverable = True
+
+    error_text = response.json().get("error", "Unknown error")
+
+    if not recoverable:
+        # fatal – abort immediately
+        return False, error_text
+
+    # recoverable – signal caller to retry without treating as error
+    return False, None
 
 
 @studio_app.command("detach")
@@ -1911,11 +1997,11 @@ def reset_studio(
                     waiter = ec2.get_waiter("volume_available")
                     waiter.wait(VolumeIds=[studio["studio_id"]])
 
-        # Reset in DynamoDB
+        # Reset in DynamoDB – align attribute names with Studio Manager backend
         table.update_item(
-            Key={"studio_id": studio["studio_id"]},
-            UpdateExpression="SET #status = :status, attached_vm_id = :vm_id, attached_device = :device",
-            ExpressionAttributeNames={"#status": "status"},
+            Key={"StudioID": studio["studio_id"]},
+            UpdateExpression="SET #st = :status, AttachedVMID = :vm_id, AttachedDevice = :device",
+            ExpressionAttributeNames={"#st": "Status"},
             ExpressionAttributeValues={
                 ":status": "available",
                 ":vm_id": None,
